// ignore_for_file: non_constant_identifier_names

import 'package:web_template/app/app_core.dart';
import 'package:web_template/widget/icon.dart';
import 'package:flutter/material.dart';
import 'package:general_lib_flutter/general_lib_flutter_core.dart';
import 'package:line_icons/line_icons.dart';

class LandingPage extends StatefulWidget {
  final AppCore appCore;

  const LandingPage({super.key, required this.appCore});
  @override
  State<LandingPage> createState() => _LandingPageState();
}

class _LandingPageState extends State<LandingPage> {
  TextEditingController usernameTextEditingController = TextEditingController();
  TextEditingController passwordTextEditingController = TextEditingController();

  GlobalKey footer_key = GlobalKey();
  final GlobalKey globalKey = GlobalKey();

  Size getRedBoxSize(GlobalKey key) {
    try {
      final box = key.currentContext!.findRenderObject() as RenderBox;
      return box.size;
    } catch (e) {
      return Size.zero;
    }
  }

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance.addPostFrameCallback((Duration duration) async {});
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      extendBodyBehindAppBar: true,
      extendBody: true,
      appBar: PreferredSize(
        preferredSize: Size(
          context.width,
          450,
        ), // here the desired height
        child: Builder(
          builder: (context) {
            Widget child = const SizedBox.shrink();
            child = MaterialButton(
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(25.0),
              ),
              hoverColor: Colors.transparent,
              splashColor: Colors.transparent,
              highlightColor: Colors.transparent,
              onPressed: () async {},
              padding: const EdgeInsets.all(15),
              child: const Row(
                mainAxisSize: MainAxisSize.max,
                mainAxisAlignment: MainAxisAlignment.start,
                children: [
                  Padding(
                    padding: EdgeInsets.all(10),
                    child: Text(
                      "General Ai App APP",
                      style: TextStyle(
                        // color: AppColors.blackColor,
                        fontSize: 16,
                        fontWeight: FontWeight.w600,
                      ),
                    ),
                  ),
                ],
              ),
            );

            return Container(
              key: globalKey,
              margin: const EdgeInsets.all(10),
              decoration: BoxDecoration(
                borderRadius: const BorderRadius.all(Radius.circular(40)),
                color: context.theme.dialogBackgroundColor.withOpacity(0.85),
                // color: Colors.white.withOpacity(0.85),
                boxShadow: [
                  BoxShadow(
                    color: context.theme.shadowColor,
                    spreadRadius: 1,
                    blurRadius: 7,
                    offset: const Offset(0, 3), // changes position of shadow
                  ),
                ],
              ),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  SizedBox(
                    height: context.mediaQueryData.padding.top,
                  ),
                  child,
                ],
              ),
            );
          },
        ),
      ),
      body: SingleChildScrollView(
        child: ConstrainedBox(
          constraints: BoxConstraints(
            minHeight: context.height,
            minWidth: context.width,
          ),
          child: Column(
            children: [
              SizedBox.fromSize(
                size: getRedBoxSize(globalKey),
              ),
              SizedBox(
                // color: Colors.green,
                width: context.width,
                height: context.height,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisAlignment: MainAxisAlignment.spaceAround,
                  children: [
                    Row(
                      children: [
                        // footer_social_media
                        Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            const Padding(
                              padding: EdgeInsets.all(10),
                              child: Text(
                                "Social Media",
                                style: TextStyle(
                                  fontWeight: FontWeight.bold,
                                ),
                              ),
                            ),
                            Padding(
                              padding: const EdgeInsets.all(5),
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: [
                                  IconWidget(
                                    iconData: LineIcons.facebook,
                                    size: 30,
                                    onPressed: () async {},
                                  ),
                                  IconWidget(
                                    iconData: LineIcons.github,
                                    size: 30,
                                    onPressed: () async {},
                                  ),
                                  IconWidget(
                                    iconData: LineIcons.instagram,
                                    size: 30,
                                    onPressed: () async {},
                                  ),
                                  IconWidget(
                                    iconData: LineIcons.telegram,
                                    size: 30,
                                    onPressed: () async {},
                                  ),
                                  IconWidget(
                                    iconData: LineIcons.youtube,
                                    size: 30,
                                    onPressed: () async {},
                                  ),
                                ],
                              ),
                            ),
                          ],
                        ),
                      ],
                    ),
                    const Align(
                      alignment: Alignment.center,
                      child: Text(
                        "Copyright 2023",
                        style: TextStyle(
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                    ),
                  ],
                ),
              ),
              Column(
                key: footer_key,
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                mainAxisSize: MainAxisSize.min,
                children: [
                  Row(
                    children: [
                      // footer_social_media
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          const Padding(
                            padding: EdgeInsets.all(10),
                            child: Text(
                              "Social Media",
                              style: TextStyle(
                                fontWeight: FontWeight.bold,
                              ),
                            ),
                          ),
                          Padding(
                            padding: const EdgeInsets.all(5),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                IconWidget(
                                  iconData: LineIcons.facebook,
                                  size: 30,
                                  onPressed: () async {},
                                ),
                                IconWidget(
                                  iconData: LineIcons.github,
                                  size: 30,
                                  onPressed: () async {},
                                ),
                                IconWidget(
                                  iconData: LineIcons.instagram,
                                  size: 30,
                                  onPressed: () async {},
                                ),
                                IconWidget(
                                  iconData: LineIcons.telegram,
                                  size: 30,
                                  onPressed: () async {},
                                ),
                                IconWidget(
                                  iconData: LineIcons.youtube,
                                  size: 30,
                                  onPressed: () async {},
                                ),
                              ],
                            ),
                          ),
                        ],
                      ),
                    ],
                  ),
                  const Align(
                    alignment: Alignment.center,
                    child: Text(
                      "Copyright 2023",
                      style: TextStyle(
                        fontWeight: FontWeight.bold,
                      ),
                    ),
                  ),
                ],
              ),
            ],
          ),
        ),
      ),
    );
  }
}
